(require [kodhy.macros [qw amap λ]])

(import
  builtins
  sys
  re
  calendar
  sqlite3
  [ast [literal-eval]]
  [collections [Counter]]
  [cheat-importing.common [commit]])

(setv dump-path "../data/dmps/am_am.dump")
(setv start-at-line 0)
; am_am has a bit less than 9,827 lines of data.

(setv cc-db-path "../data/amcc.sqlite")
(setv target-db-path "/dev/shm/amam.sqlite") ;"../data/amam.sqlite")

(setv gender-codes-to-female-flag {1 1  2 0})

;; ------------------------------------------------------------
;; * Setup
;; ------------------------------------------------------------

(setv amcc (sqlite3.connect cc-db-path))
(setv amcc-usernums (frozenset (map first
  (.execute amcc "select distinct usernum from T"))))
(.close amcc)
(setv amcc-usernums-str (frozenset (map str amcc-usernums)))

(setv target-db (sqlite3.connect target-db-path))

(setv colnames (qw id createdon createdby updatedon updatedby admin status account_type membership_status ad_source profile_number nickname first_name last_name street1 street2 city zip state latitude longitude country phone work_phone mobile_phone gender dob profile_caption profile_ethnicity profile_weight profile_height profile_bodytype profile_smoke profile_drink profile_initially_seeking profile_relationship pref_opento pref_opento_other pref_opento_abstract pref_turnsmeon pref_turnsmeon_other pref_turnsmeon_abstract pref_lookingfor pref_lookingfor_other pref_lookingfor_abstract main_photo security_question security_answer))

(unless (.fetchall (.execute target-db "select name from sqlite_master where type='table' and name='Users'"))
  (.execute target-db "pragma journal_mode = wal")
  (.execute target-db "pragma foreign_keys = on")
  (.execute target-db "create table Users (
      usernum integer primary key,
      created integer not null,
      updated integer not null,
      female integer,
      born integer not null,
        -- A date in the form YYYYmmdd.
      ethnicity integer,
      smoking integer,
      drinking integer,
      seeking integer)")
  (for [table-name (qw OpenTo TurnsMeOn LookingFor)]
    (.execute target-db (+ "create table " table-name " (
        usernum integer not null
            references Users(usernum),
        code integer not null,
        primary key (usernum, code))"))))

;; ------------------------------------------------------------
;; * Main loop for data processing
;; ------------------------------------------------------------

(setv to-insert (dfor
  [x] (.execute target-db "select name from sqlite_master where type='table'")
  [x []]))

(setv line-prefix "INSERT INTO `am_am_member` VALUES ")

(with [o (open dump-path "rt" :encoding "UTF-8" :errors "backslashreplace")]
  (for [[line-i line] (enumerate o)]
    (unless (and (>= line-i start-at-line) (.startswith line line-prefix))
      (continue))
    (print (format line-i ",") :end " ")
    (.flush sys.stdout)

    ; Optimization: skip this line if there's no good usernum there.
    (unless (any (gfor x (re.findall "\((\d+)," line) (in x amcc-usernums-str)))
      (continue))

    (for [profile (builtins.eval
        (cut line (len line-prefix) -2)
        {"NULL" None})]
      (setv profile (dict (zip colnames profile)))
      (unless (in (get profile "id") amcc-usernums)
        (continue))

      (.append (get to-insert "Users") (dict
         :usernum (get profile "id")
         :created (calendar.timegm (amap (int it) (re.findall r"\d+" (get profile "createdon"))))
         :updated (calendar.timegm (amap (int it) (re.findall r"\d+" (get profile "updatedon"))))
         :female (.get gender-codes-to-female-flag (get profile "gender"))
         :born (int (.join "" (.split (get profile "dob") "-")))
         :ethnicity (get profile "profile_ethnicity")
         :smoking (get profile "profile_smoke")
         :drinking (get profile "profile_drink")
         :seeking (get profile "profile_relationship")))
      (for [table-name (qw OpenTo TurnsMeOn LookingFor)]
        (for [code (map int (filter identity (.split
            (get profile (+ "pref_" (.lower table-name))) "|")))]
          (.append (get to-insert table-name) (dict
            :usernum (get profile "id")
            :code code)))))

    (when (or
        (zero? (% line-i 100))
        (>= (len (get to-insert "Users")) 10,000))
      (print "Queued:" (format (len (get to-insert "Users")) ","))
      (commit target-db to-insert))))

(commit target-db to-insert)
(.execute target-db "pragma journal_mode = delete")
(.close target-db)
