(require [kodhy.macros [qw filt amap block]])

(import
  re
  [unidecode [unidecode]]
  [kodhy.util [all-unique? ret unique]])

(defn hyphenated-int [x]
  (setv x (.replace x "-" ""))
  (if x
    (int x)
    None))

(defn fits-sqlite-int [x]
  ; SQLite uses signed 64-bit integers.
  (<= (** -2 63) x (dec (** 2 63))))

(setv states (qw AL AK AZ AR CA CO CT DE FL GA HI ID IL IN IA KS KY LA ME MD MA MI MN MS MO MT NE NV NH NJ NM NY NC ND OH OK OR PA RI SC SD TN TX UT VT VA WA WV WI WY))
(setv state-names (dict :AL "Alabama" :AK "Alaska" :AZ "Arizona" :AR "Arkansas" :CA "California" :CO "Colorado" :CT "Connecticut" :DE "Delaware" :FL "Florida" :GA "Georgia" :HI "Hawaii" :ID "Idaho" :IL "Illinois" :IN "Indiana" :IA "Iowa" :KS "Kansas" :KY "Kentucky" :LA "Louisiana" :ME "Maine" :MD "Maryland" :MA "Massachusetts" :MI "Michigan" :MN "Minnesota" :MS "Mississippi" :MO "Missouri" :MT "Montana" :NE "Nebraska" :NV "Nevada" :NH "New Hampshire" :NJ "New Jersey" :NM "New Mexico" :NY "New York" :NC "North Carolina" :ND "North Dakota" :OH "Ohio" :OK "Oklahoma" :OR "Oregon" :PA "Pennsylvania" :RI "Rhode Island" :SC "South Carolina" :SD "South Dakota" :TN "Tennessee" :TX "Texas" :UT "Utah" :VT "Vermont" :VA "Virginia" :WA "Washington" :WV "West Virginia" :WI "Wisconsin" :WY "Wyoming"))
(setv states (tuple (map first (sorted (.items state-names) :key second))))
  ; So `states` is a tuple of state abbreviations, but sorted in
  ; order of what they stand for rather than in order of the
  ; abbreviations themselves.
(setv state-pop-2012 (dict :AL 4822023 :AK 731449 :AZ 6553255 :AR 2949131 :CA 38041430 :CO 5187582 :CT 3590347 :DE 917092 :FL 19317568 :GA 9919945 :HI 1392313 :ID 1595728 :IL 12875255 :IN 6537334 :IA 3074186 :KS 2885905 :KY 4380415 :LA 4601893 :ME 1329192 :MD 5884563 :MA 6646144 :MI 9883360 :MN 5379139 :MS 2984926 :MO 6021988 :MT 1005141 :NE 1855525 :NV 2758931 :NH 1320718 :NJ 8864590 :NM 2085538 :NY 19570261 :NC 9752073 :ND 699628 :OH 11544225 :OK 3814820 :OR 3899353 :PA 12763536 :RI 1050292 :SC 4723723 :SD 833354 :TN 6456243 :TX 26059203 :UT 2855287 :VT 626011 :VA 8185867 :WA 6897012 :WV 1855413 :WI 5726398 :WY 576412))
  ; https://www2.census.gov/programs-surveys/popest/tables/2010-2012/state/totals/nst-est2012-01.xls
(setv available-states (qw CA FL KS NY OK))

(defn state-code [abbr] (* 1,000,000,000,000 (.index states abbr)))
(defn next-state-code [abbr] (+ 1,000,000,000,000 (state-code abbr)))

(setv voter-fields (dict

  :CA
    ["Locality Code" "Registrant ID" "Last Name" "First Name" "Middle Name" "Name Suffix" "Addr. Num." "Addr. Num. Suff." "St. Dir. Prefix" "Street Name" "Street Type" "St. Dir. Suffix" "Unit Type" "Unit Number" "City" "State" "Zip" "Telephone (Area Code)" "Telephone Exchange" "Telephone Number" "Mailing Address1" "Mailing Address2" "Mailing City" "Mailing State" "Mailing Zip" "Language" "Date of Birth" "Gender" "Party" "Status" "Status Reason" "Registration Date" "Precinct" "Precinct Part" "Registration Method Code" "Assistance Flag" "Place of Birth" "Section Township Range Direction" "Previous Registration ID" "Previous County Code" "Previous Last Name" "Previous First Name" "Previous Middle name" "Previous Name Suffix" "Previous Residence Street Number" "Previous Residence Street Number Suffix" "Previous Residence Street Name" "Previous Residence Street Direction Prefix" "Previous Residence Street Direction Suffix" "Previous Residence Street Type" "Previous Residence Unit Type" "Previous Residence Unit Number" "Previous Residence City" "Previous Residence State" "Previous Residence Zip" "Name Prefix" "Previous Name Prefix" "Dist1" "Dist2" "Dist3" "Dist4" "Dist5" "Dist6" "Dist7" "Dist8" "Elec1" "Elec2" "Elec3" "Elec4" "Elec5" "Elec6" "Elec7" "Elec8" "Non Standard Address"]

  :FL
    ["County Code" "Voter ID" "Name Last" "Name Suffix" "Name First" "Name Middle" "Suppress Address, DOB and Phone Number" "Residence Address Line 1" "Residence Address Line 2" "Residence City (USPS)" "Residence State" "Residence Zipcode" "Mailing Address Line 1" "Mailing Address Line 2" "Mailing Address Line 3" "Mailing City" "Mailing State" "Mailing Zipcode" "Mailing Country" "Gender" "Race" "Birth Date" "Registration Date" "Party Affiliation" "Precinct" "Precinct Group" "Precinct Split" "Precinct Suffix" "Voter Status" "Congressional District" "House District" "Senate District" "County Commission District" "School Board District" "Daytime Area Code" "Daytime Phone Number" "EOL"]

  :KS
    ["db_logid" "cde_registrant_status" "cde_name_title" "text_name_first" "text_name_middle" "text_name_last" "cde_name_suffix" "cde_gender" "text_registrant_id" "text_res_address_nbr" "text_res_address_nbr_suffix" "cde_street_dir_prefix" "text_street_name" "cde_street_type" "cde_street_dir_suffix" "cde_res_unit_type" "text_res_unit_nbr" "text_res_city" "cde_res_state" "text_res_zip5" "text_res_zip4" "text_res_carrier_rte" "ind_res_military" "text_mail_address1" "text_mail_address2" "text_mail_address3" "text_mail_address4" "text_mail_city" "cde_mail_state" "text_mail_zip5" "text_mail_zip4" "text_mail_carrier_rte" "ind_mail_military" "ind_mail_foreign" "desc_party" "precinct_text_name" "precinct_text_designation" "date_of_birth" "date_of_registration" "text_phone_area_code" "text_phone_exchange" "text_phone_last_four" "text_election_code_1" "text_election_code_2" "text_election_code_3" "text_election_code_4" "text_election_code_5" "text_election_code_6" "text_election_code_7" "text_election_code_8" "text_election_code_9" "text_election_code_10" "cde_party" "ind_qualified" "district_cg" "district_jd" "district_kr" "district_ks" "district_sb" "district_pt" "district_sd"]

  :NY
    ["lastname" "firstname" "middlename" "namesuffix" "raddnumber" "rhalfcode" "rapartment" "rpredirection" "rstreetname" "rpostdirection" "rcity" "rzip5" "rzip4" "mailadd1" "mailadd2" "mailadd3" "mailadd4" "dob" "gender" "enrollment" "otherparty" "countycode" "ed" "ld" "towncity" "ward" "cd" "sd" "ad" "lastvoteddate" "prevyearvoted" "prevcounty" "prevaddress" "prevname" "countyvrnumber" "regdate" "vrsource" "idrequired" "idmet" "status" "reasoncode" "inact_date" "purge_date" "sboeid" "VoterHistory"]

  :OK
    ["Precinct number" "Last name" "First name" "Middle name" "Suffix" "Voter identification number" "Political affiliation" "Status" "Street/house number" "Street direction" "Street name" "Street type" "Building number" "City/municipality" "ZIP Code" "Birth date" "Date of registration" "Mailing street address 1" "Mailing street address 2" "Mailing address city" "Mailing address state" "Mailing address ZIP Code" "Election date" "Method of voting"]))

(setv parties (qw

  np  ; No party affiliation

  ; New York codes
  DEM ; Democratic
  REP ; Republican
  CON ; Conservative
  WOR ; Working Families
  IND ; Independence
  GRE ; Green
  RTH ; Rent Is Too High
  LBT ; Libertarian
  APP ; Antiprohibition
  FDM ; Freedom Party
  TXP ; Taxpayers
  SWP ; Not documented

  ; Florida codes
  AIP ; American's Party of Florida 
  CPF ; Constitution Party of Florida 
  ;DEM ; Florida Democratic Party 
  ECO ; Ecology Party of Florida 
  FPP ; Florida Pirate Party
  FSW ; Florida Socialist Workers Party
  ;GRE ; Green Party of Florida
  IDP ; Independence Party of Florida 
    ; http://ipfl.org
    ; Doesn't seem to be affiliated with the Independence Party (NY).
  INT ; Independent Party of Florida 
  JPF ; Justice Party of Florida
  PSL ; Party for Socialism and Liberation - Florida
  PFP ; Peace & Freedom Party of Florida
  REF ; Reform Party 
  ;REP ; Republican Party of Florida 
  TPF ; Tea Party of Florida
  OBJ ; Not documented
  UNK ; Not documented
  NRS ; Not documented
  AEL ; Not documented
  FWP ; Not documented
  SOC ; Not documented

  ; California codes
  ; For hints, I used:
  ; - http://elections.cdn.sos.ca.gov/ccrov/pdf/2010/december/10329jcg.pdf
  ; - http://elections.cdn.sos.ca.gov/ror/ror-pages/15day-general-12/qual-pol-parties1.pdf
  AI ; American Independent Party 
  ;DEM ; Democratic Party
  PF ; Peace and Freedom Party 
  ;REP ; Republican Party 
  DS ; Decline to State 
  OTH ; Other
  ; A lot of undocumented parties
  MIS AME NAT JP CTP CPC MCP SAP WP
  WWP ; (perhaps We Like Women Party)
  HUM MMW FED
  PIR ; (perhaps California Pirate Party)
  HPC LRU OP CP CMP NRP PPC POT NMB DP AMC
  CPP ; (perhaps California Pirate Party)
  GSP TVP SEU EGA UCB ACP NSP EJP U08 ATP UCA UMP WFP

  ; Oklahoma codes (none documented)
  ; DEM
  ; REP
  ; IND
  AE

))

(assert (all-unique? parties))

(setv party-aliases {
  ""    ; Presumably means no party
    "np"
  "BLK" ; New York code for no party affiliation
    "np"
  "LPF" ; Libertarian Party of Florida
    "LBT"
  "NPA" ; Undocumented, but presumably stands for "no party affiliation"
    "np"
  "NPP" ; No Party Preference (California)
    "np"
  "GRN" ; Green Party (California)
    "GRE"
  "LIB" ; Libertarian Party (California) and an undocumented party in Florida
    "LBT"

  ; Kansas uses names instead of 3-letter codes.
  "Unaffiliated"
    "np"
  "Democratic"
    "DEM"
  "Republican"
    "REP"
  "Libertarian"
    "LBT"})

(assert (not (& (set (.keys party-aliases)) (set parties))))
(assert (.issubset (set (.values party-aliases)) (set parties)))

(setv party-codes (dfor [i p] (enumerate parties) [p i]))
(for [[k v] (.items party-aliases)]
  (setv (get party-codes k) (get party-codes v)))

(setv keep-parties (qw np DEM REP GRE LBT))

(setv long-party-names (dict
  :np  "unaffiliated"
  :DEM "Democratic"
  :REP "Republican"
  :GRE "Green"
  :LBT "Libertarian"))

(setv born-restrictions (dict
  ; These are dates that either are impossible (like September
  ; 31) or were provided by an implausibly large number of
  ; voters. See the Cheat notebook for details.
  :CA "born > 19010101 and born != 19310101"
  :FL "born != 19421010"
  :KS "1"
  :NY "born != 19500101 and born != 19210101"
  :OK "born not in (19080000, 19100000, 19110000, 19130000, 19140000, 19160000, 19170000, 19180000, 19190000, 19200000, 19210000, 19220000, 19230000, 19240000, 19250000, 19260000, 19270000, 19280000, 19290000, 19300000, 19310000, 19320000, 19330000, 19340000, 19340931, 19350000, 19360000, 19370000, 19380000, 19390000, 19400000, 19410000, 19420000, 19430000, 19450000, 19460100, 19470000, 19480000, 19480014, 19490000, 19510000, 19530000, 19550000, 19640000)"))

(setv amcc-fields (,
  ; On the right are example values (some of which are fake, so
  ; this file doesn't contain personally identifiable
  ; information).
  "ACCOUNT"            ; "99926056"
  "ACCOUNT NAME"       ; "Avid Dating Life Inc"
  "AMOUNT"             ; "68.99"
  "AUTH CODE"          ; "05166Z"
  "AVS"                ; "Z"
  "BRAND"              ; "MC"
  "CARD ENDING"        ; "1337"
  "CVD"                ; "M"
  "USERNUM"            ; "46173585"
    ; Actually labeled "FIRST NAME".
  "NAME"               ; "JOHN G PUBLIC"
    ; Actually labeled "LAST NAME".
  "MERCHANT TRANS. ID" ; "66337834"
  "OPTION CODE"        ; ""
  "DATE"               ; "2000-01-01 00:48:22"
  "TXN_ID"             ; "5554665651"
  "CONF_NO"            ; "5554665651"
  "ERROR_CODE"         ; ""              See http://web.archive.org/web/20160512130655/https://support.mindbodyonline.com/hc/en-us/articles/203255433-Optimal-Payments-credit-card-decline-codes
  "AUTH TYPE"          ; "Undefined"
  "TYPE"               ; "Authorizations"
  "TXT_CITY"           ; "Salt Lake City"
  "TXT_COUNTRY"        ; "US"
  "TXT_EMAIL"          ; "HUNTER2@EXAMPLE.COM"
  "TXT_PHONE"          ; ""
  "TXT_STATE"          ; "UT"
  "TXT_ADDR1"          ; ""
  "TXT_ADDR2"          ; ""
  "ZIP"                ; "84105"
  "CONSUMER_IP"        ; "172.16.254.1"
))

(setv epoch-year 2012)

(defn digest-personal-name [s one-word]
  (setv s
    ; Remove hyphens and apostrophes.
    (re.sub "[-']" ""
    ; Strip suffixes.
    (re.sub r"[.\s]\s*(I|II|III|JR|SR)\.?\Z" ""
    ; Strip leading initials.
    (re.sub r"\A([A-Z][.\s]\s*)+" ""
    ; Strip prefixes.
    (re.sub r"\A(MR|MS|MRS|MISS|DR)[.\s]\s*" ""
    (unidecode
    (.strip
    (.upper
    s))))))))
  (if one-word
    ; Get the longest (or first) run of letters.
    (do
      (setv ms (re.findall "[A-Z]+" s))
      (if ms (max ms :key len) ""))
    ; Translate all other non-letters to spaces.
    (.strip (re.sub "[^A-Z]+" " " s))))

(defn digest-zipcode [zipcode] (block
  (setv zipcode (cut (.replace (.replace zipcode "-" "") " " "") 0 5))
  (unless (.isdigit zipcode)
    ; Not a real ZIP code (maybe a postal code for another
    ; country).
    (ret))
  (setv zipcode (int zipcode))
  (when (< zipcode 501) ; The smallest real ZIP code.
    (ret))
  zipcode))

(defn get-addrnums [addrtext zipcode]
  (setv zipcode (str zipcode))
  (setv addrnums (list
    (map int
    ; But exclude those that are very long or begin with the ZIP code.
    (filt (not (or (> (len it) 16) (.startswith it zipcode)))
    ; Sort by length (breaking ties by original order).
    (sorted :key len :reverse True
    ; Keep the unique ones.
    (unique
    ; Find all numerals, skipping leading 0s (or all-0 numerals).
    (re.findall "[1-9][0-9]*"
    ; Remove hyphens and apostrophes.
    (re.sub "[-']" ""
    (unidecode addrtext)))))))))
  (setv addrnum1 (when addrnums (.pop addrnums 0)))
  (setv addrnum2 (when addrnums (first addrnums)))
  [addrnum1 addrnum2])

(defn commit [db to-insert]
  (unless (instance? dict to-insert)
    (setv to-insert {"T" to-insert}))
  (when (any (.values to-insert))
    (print "Inserting")
    (for [[table-name rows] (.items to-insert)]
      (setv sql-string (.format "insert or ignore into {} ({}) values ({})"
        table-name
        (.join ", " (.keys (first rows)))
        (.join ", " (amap (+ ":" it) (.keys (first rows))))))
      (.executemany db sql-string rows)))
  (print "Committing")
  (.commit db)
  (print "Committed")
  (for [rows (.values to-insert)]
    (del (cut rows))))
