; This script sets the am_user column of the output of voter-csv-to-sqlite.hy.

; N.B. If possible, make sure the databases are entirely loaded
; into physical RAM (with, e.g., "wc -l DATABASE") before running
; this script.

(require [kodhy.macros [amap filt λ]])

(import
  re
  [itertools [groupby]]
  [collections [defaultdict]]
  sqlite3
  [unidecode [unidecode]]
  [cheat-importing.common [hyphenated-int]])

(setv cc-db-path "../data/amcc.sqlite")
(setv voter-db-path "../data/voter.sqlite")
(setv update-every 1,000)
(setv ambiguous-am-user-code -1)

(setv db (sqlite3.connect cc-db-path))
(.execute db "attach ? as V" (, voter-db-path))
(setv db.row-factory sqlite3.Row)

(print "Starting loop")
(setv to-add [])
(setv assigned-voters (set))
(for [[user-n [usernum xacts]] (enumerate (groupby
    (.execute db "select * from T
      where zip in (select zip from V.T)
      order by usernum, time desc")
      ; The "where" clause here is just an optimization.
    (λ (get it "usernum"))))]

  (when (= (% user-n update-every) 0)
    (print (.format "{:,} - {}" user-n usernum)))

  (for [xact xacts]
    ; Because we used "order by … time desc", we'll be looking at
    ; the more recent transactions first.

    (setv voter-rows (.fetchall (.execute db
      "select * from V.T where first_name = ? and last_name = ? and zip = ?"
      [(get xact "first_name") (get xact "last_name") (get xact "zip")])))
    (setv addr-matches (filt
      (.intersection
        (set (+ [(get it "addrnum1")]
          (if (none? (get it "addrnum2")) [] [(get it "addrnum2")])))
        (set (+ [(get xact "addrnum1")]
          (if (none? (get xact "addrnum2")) [] [(get xact "addrnum2")]))))
      voter-rows))
    (when (= (len addr-matches) 1)
      (setv voter-rn (get addr-matches 0 "rn"))
      (print (get xact "usernum") voter-rn
        (get xact "first_name") (get xact "last_name") (get xact "zip")
        (in voter-rn assigned-voters))
      ; We use assigned-voters to make sure that if a voter
      ; matches more than one AM user, the am_user column is set to
      ; ambiguous-am-user-code instead of a specific AM user.
      (.append to-add (,
        (if (in voter-rn assigned-voters)
          ambiguous-am-user-code
          (get xact "usernum"))
        voter-rn))
      (.add assigned-voters voter-rn)
      (break))))

(print "Updating")
(.executemany db "update V.T set am_user = ? where rn = ?" to-add)
(print "Committing")
(.commit db)
(print "Committed")
