(require [kodhy.macros [rmap amap]])
(import [hy.contrib.hy-repr [hy-repr]])

(import
  os.path
  json
  sqlite3
  [cheat-importing.common [*]])

(defn db-path [basename]
  (os.path.join "../data" basename))
(setv output-path "../summary.json")

(setv output {})

(print "Main table")
(setv (get output "main_table") (do
  (setv db (sqlite3.connect (db-path "voter.sqlite")))
  (.execute db "attach ? as PDO" (, (db-path "paid_delete_only.sqlite")))
  (setv db.row-factory sqlite3.Row)
  (assert (>= sqlite3.sqlite-version-info (, 3 8 3)))
    ; SQLite 3.8.3 is needed for WITH clauses.
  (setv rows (list (.execute db (.format
    :epoch-year epoch-year
    :state-cases (.join " " (rmap
      [[state expr] (sorted (.items born-restrictions))]
      (.format "when {} then {}" (.index states state) expr)))
    "with TS as (select rn / cast(1e12 as integer) as state, * from T)
      select
         state,
         female,
         gender_is_imputed,
         case when (born between 19000102 and 20000000) and
                 born != 19010101 and
                 (case state {state_cases} end) and
                 ((registered isnull) or born < registered)
             then {epoch_year} - born / 10000
             else null end as age_years,
         party,
         count(*) as voters,
         sum(am_user notnull and PDO.T.usernum isnull) as am
      from TS left outer join PDO.T on TS.am_user = PDO.T.usernum
      group by state, female, gender_is_imputed, age_years, party
      order by state, female, gender_is_imputed, age_years, party"))))
  {"columns" (.keys (first rows)) "data" (amap (tuple it) rows)}))

(print "AM user counts dictionary")
(setv (get output "am_user_counts") (do
  (setv db (sqlite3.connect (db-path "amcc.sqlite")))
  (.execute db "attach ? as Z" (, (db-path "zipstates.sqlite")))
  (.execute db "attach ? as PDO" (, (db-path "paid_delete_only.sqlite")))
  (setv x {})
  (for [state available-states]
    (print state)
    (setv (get x state) (first (first (.execute db
       "select count(distinct usernum)
       from T
       where zip in (select zipcode from Z.T where state = ?)
         and usernum not in (select usernum from PDO.T)"
       [(.index states state)])))))
  x))

(print "Writing output")
(with [o (open output-path "w")]
  (json.dump output o :sort-keys True :separators ["," ":"]))
