; Make a database of AM users who only made one credit-card
; transaction and who paid to delete their account. These users
; seem to have done nothing on AM except delete their account.

(require [kodhy.macros [amap λ]])

(import
  os
  sys
  re
  sqlite3)

(setv dump-path-fmt "../dmps/{}.dump")
(setv target-db-path "../data/paid_delete_only.sqlite")
(setv cc-db-path "../data/amcc.sqlite")

; aminno_member has a bit less than 13,271 lines of data.
; am_am has a bit less than 9,827.

;; ------------------------------------------------------------
;; * Read the MySQL dumps
;; ------------------------------------------------------------

(setv paid-delete-ids (set))

(for [orig-db ["am_am" "aminno_member"]]
  (defn msg [x] (print (+ orig-db ":") x))
  (setv line-prefix (.format "INSERT INTO `{}` VALUES ("
    (if (= orig-db "am_am") "am_am_member" orig-db)))
  (setv line-prefix (.encode line-prefix "ASCII"))

  (msg "Opening dump file")
  (with [o (open (.format dump-path-fmt orig-db) "rb")]
    (msg "Opened")

    (for [[line-n line] (enumerate o)]
      (unless (.startswith line line-prefix)
        (continue))
      (when (= (% line-n 500) 0)
        (msg (.format "Line {:6,} - got {:9,}" line-n (len paid-delete-ids))))

      (.update paid-delete-ids
        (amap (int (cut it None None -1))
        (re.findall rb"'>eteled_diap<',.+?,(\d+)\(,\)"
        (cut line None None -1)))))))

;; ------------------------------------------------------------
;; * Create the new database
;; ------------------------------------------------------------

(try (os.remove target-db-path) (except [OSError]))
(setv target-db (sqlite3.connect target-db-path))
(.execute target-db "pragma foreign_keys = on")
(.execute target-db "create table T(usernum integer primary key)")
(.execute target-db "attach ? as CC" (, cc-db-path))

(print "Inserting")
(.executemany target-db "insert into T values (?)"
  (map (λ (, it)) paid-delete-ids))

(print "Culling")
; We keep only users who have exactly one credit-card transaction.
(.execute target-db "delete from T where usernum not in
  (select usernum
    from (select usernum, count(*) as n from CC.T group by usernum)
    where n = 1)")

(print "Committing")
(.commit target-db)

(print "Vacuuming")
(.execute target-db "vacuum")

(.close target-db)
