(require [kodhy.macros [
  λ $ qw amap rmap filt getl geti ss cbind ordf wc cached]])

(import
  re
  random
  os
  atexit
  json
  [collections [Counter OrderedDict]]
  sqlite3
  [numpy :as np]
  [pandas :as pd]
  [kodhy.util [T F rd thousep ilogit pairs cached-eval
    pds-from-pairs df-from-pairs drop-unused-cats keyword->str]]
  [cheat-importing.common [*]])

(setv cache-dir (os.path.join (os.path.expanduser "~")
  ".daylight" "py-cache" "Kodi-projects" "cheat"))
(defmacro cache [expr &optional [bypass 'None]]
  `(cached ~expr ~bypass (os.path.join (os.path.expanduser "~")
     ".daylight" "py-cache" "Kodi-projects" "cheat")))

(setv amam (sqlite3.connect "../data/amam.sqlite"))
(atexit.register (fn [] (.close amam)))
(setv codes (with [o (open "amcodes.json")] (json.load o)))

(setv summary (with [o (open "../summary.json")] (json.load o)))

(setv counts (pd.DataFrame (get summary "main_table" "data")
  :columns (get summary "main_table" "columns")))
(setv am-users-in-states (get summary "am_user_counts"))

(setv counts (wc counts (cbind
  :state (do
    (setv v (.astype $state "category"))
    (.rename-categories v.cat (amap (get states it) v.cat.categories)))
  :gender (pd.Categorical.from-codes (.fillna $female -1) ["Male" "Female"])
  :gender_known (& (.notnull $female) (= $gender_is_imputed 0))
  :age $age_years
  :party (pd.Categorical.from-codes $party parties)
  $voters
  :am_users $am)))

(defn user-var-freq [vname]
  (setv d (pd.read-sql :con amam (.format
    "select {}, count(*) as n from Users group by {}"
    vname vname)))
  (setv (getl d : vname) (.fillna :value "N/A" (.map (getl d : vname)
    (dfor [k v] (.items (get codes vname))
      [(int k) v]))))
  (setv d (ordf (.sum (.groupby d vname)) (- $n)))
  d)

(defn msf-prefcode-freq [table-name]
  (setv seeking (str (tuple (gfor
    [k v] (.items (get codes "seeking"))
    :if (re.search r"\bmale seeking female" v)
    (int k)))))
  (setv n-people (first (first (.execute amam (+
    "select count(*) from Users where seeking in " seeking)))))
  (setv d (pd.read-sql :con amam (.format
    "select code, count(*) as n
        from {}
        where usernum in (select usernum from Users where seeking in {})
        group by code"
    table-name seeking)))
  (setv ($ d code) (.fillna :value "N/A" (.map ($ d code)
    (dfor [k v] (.items (get codes table-name))
      [(int k) (get v "val")]))))
  (setv d (ordf (.sum (.groupby d "code")) (- $n)))
  (setv d (cbind d :p (rd (/ ($ d n) n-people))))
  (setv ($ d n) (thousep ($ d n)))
  d)
